@@ -8,12 +8,12 @@ support.
 
 __author__ = "Marcel Rieger"
 __email__ = "python-scinum@googlegroups.com"
-__copyright__ = "Copyright 2017-2020, Marcel Rieger"
+__copyright__ = "Copyright 2017-2021, Marcel Rieger"
 __credits__ = ["Marcel Rieger"]
 __contact__ = "https://github.com/riga/scinum"
 __license__ = "BSD-3-Clause"
 __status__ = "Development"
-__version__ = "1.1.1"
+__version__ = "1.1.3"
 __all__ = ["Number", "Operation", "ops", "style_dict", "REL", "ABS", "NOMINAL", "UP", "DOWN"]
 
 
@@ -23,6 +23,7 @@ import functools
 import operator
 import types
 import decimal
+from collections import defaultdict
 
 # optional imports
 try:
@@ -32,6 +33,13 @@ except ImportError:
     np = None
     HAS_NUMPY = False
 
+try:
+    import uncertainties as _uncs
+    HAS_UNCERTAINTIES = True
+except ImportError:
+    _uncs = None
+    HAS_UNCERTAINTIES = False
+
 
 # version related adjustments
 string_types = (str,)
@@ -300,6 +308,14 @@ class Number(object):
         # numpy settings
         self.dtype = np.float32 if HAS_NUMPY else None
 
+        # prepare conversion from uncertainties.ufloat
+        if is_ufloat(nominal):
+            # uncertainties must not be set
+            if uncertainties:
+                raise ValueError("uncertainties must not be set when converting a ufloat")
+            # extract nominal value and uncertainties
+            nominal, uncertainties = parse_ufloat(nominal)
+
         # set initial values
         self.nominal = nominal
         if uncertainties is not None:
@@ -440,13 +456,26 @@ class Number(object):
         uncertainties = self.__class__.uncertainties.fparse(self, {name: value})
         self._uncertainties.update(uncertainties)
 
+    def clear(self, nominal=None, uncertainties=None):
+        """
+        Removes all uncertainties and sets the nominal value to zero (float). When *nominal* and
+        *uncertainties* are given, these new values are set on this instance.
+        """
+        self.uncertainties = {}
+        self.nominal = 0.
+
+        if nominal is not None:
+            self.nominal = nominal
+        if uncertainties is not None:
+            self.uncertainties = uncertainties
+
     def str(self, format=None, unit=None, scientific=False, si=False, labels=True, style="plain",
             styles=None, force_asymmetric=False, **kwargs):
         r"""
         Returns a readable string representiation of the number. *format* is used to format
         non-NumPy nominal and uncertainty values. It can be a string such as ``"%d"``, a function
-        that is called with the value to format, or a rounding method as accepted by
-        :py:meth:`round_value`. When *None* (the default), :py:attr:`default_format` is used. All
+        that is called with the value to format, or a rounding function as accepted by
+        :py:func:`round_value`. When *None* (the default), :py:attr:`default_format` is used. All
         keyword arguments except wildcard *kwargs* are only used to format non-NumPy values. In case
         of NumPy objects, *kwargs* are passed to `numpy.array2string
         <https://docs.scipy.org/doc/numpy/reference/generated/numpy.array2string.html>`_.
@@ -517,6 +546,10 @@ class Number(object):
 
             # special formats implemented by round_value
             if format in ("pub", "publication", "pdg", "one", "onedigit"):
+                # complain when no uncertainties exist
+                if not ups:
+                    raise Exception("cannot apply format '{}' when no uncertainties exist".format(
+                        format))
                 nominal, (ups, downs), _mag = round_value(self.nominal, ups, downs, method=format)
                 fmt = lambda x: match_precision(float(x) * 10.**_mag, 10.**_mag)
 
@@ -593,7 +626,8 @@ class Number(object):
         if not self.is_numpy:
             text = "'" + self.str(*args, **kwargs) + "'"
         else:
-            text = "{} numpy array, {} uncertainties".format(self.shape, len(self.uncertainties))
+            text = "numpy array, shape {}, {} uncertainties".format(self.shape,
+                len(self.uncertainties))
 
         return "<{} at {}, {}>".format(self.__class__.__name__, hex(id(self)), text)
 
@@ -708,7 +742,7 @@ class Number(object):
         uncs = {}
         default = (0., 0.)
         for name in set(num.uncertainties.keys()) | set(other.uncertainties.keys()):
-            _rho = rho if not isinstance(rho, dict) else rho.get(name, 1.)
+            _rho = rho.get(name, 1.) if isinstance(rho, dict) else rho
 
             num_unc = num.get_uncertainty(name, default=default)
             other_unc = other.get_uncertainty(name, default=default)
@@ -717,11 +751,34 @@ class Number(object):
                 nom1=num.nominal, nom2=other.nominal, rho=_rho) for i in range(2))
 
         # store values
-        num.nominal = nom
-        num.uncertainties = uncs
+        num.clear(nom, uncs)
 
         return num
 
+    def __array_ufunc__(self, ufunc, method, *inputs, **kwargs):
+        # only direct calls of the ufunc are supported
+        if method != "__call__":
+            return NotImplemented
+
+        # try to find the proper op for that ufunc
+        op = ops.get_ufunc_operation(ufunc)
+        if op is None:
+            return NotImplemented
+
+        # extract kwargs
+        out = kwargs.pop("out", None)
+
+        # apply the op
+        result = op(*inputs, **kwargs)
+
+        # insert in-place to an "out" object?
+        if out is not None:
+            out = out[0]
+            out.clear(result.nominal, result.uncertainties)
+            result = out
+
+        return result
+
     def __call__(self, *args, **kwargs):
         # shorthand for get
         return self.get(*args, **kwargs)
@@ -879,14 +936,21 @@ class Operation(object):
        read-only
 
        The name of the operation.
+
+    .. py:attribute:: ufuncs
+       type: list
+       read-only
+
+       List of ufunc objects that this operation handles.
     """
 
-    def __init__(self, function, derivative=None, name=None):
+    def __init__(self, function, derivative=None, name=None, ufuncs=None):
         super(Operation, self).__init__()
 
         self.function = function
         self.derivative = derivative
         self._name = name or function.__name__
+        self._ufuncs = ufuncs or []
 
         # decorator for setting the derivative
         def derive(derivative):
@@ -898,9 +962,35 @@ class Operation(object):
     def name(self):
         return self._name
 
+    @property
+    def ufuncs(self):
+        return self._ufuncs
+
     def __repr__(self):
         return "<{} '{}' at {}>".format(self.__class__.__name__, self.name, hex(id(self)))
 
+    def __call__(self, num, *args, **kwargs):
+        if self.derivative is None:
+            raise Exception("cannot run operation '{}', no derivative registered".format(
+                self.name))
+
+        # ensure we deal with a number instance
+        num = ensure_number(num)
+
+        # apply to the nominal value
+        nominal = self.function(num.nominal, *args, **kwargs)
+
+        # apply to all uncertainties via
+        # unc_f = derivative_f(x) * unc_x
+        x = abs(self.derivative(num.nominal, *args, **kwargs))
+        uncertainties = {}
+        for name in num.uncertainties:
+            up, down = num.get_uncertainty(name)
+            uncertainties[name] = (x * up, x * down)
+
+        # create and return the new number
+        return num.__class__(nominal, uncertainties)
+
 
 class OpsMeta(type):
 
@@ -920,10 +1010,14 @@ class ops(with_metaclass(OpsMeta, object)):
         print(num) # -> 25.00 (+10.00, -10.00)
     """
 
+    # registered operations mapped to their names
     _instances = {}
 
+    # mapping of ufunc to operation names for faster lookup
+    _ufuncs = {}
+
     @classmethod
-    def register(cls, function=None, name=None):
+    def register(cls, function=None, name=None, ufunc=None):
         """
         Registers a new math function *function* with *name* and returns an :py:class:`Operation`
         instance. A math function expects a :py:class:`Number` as its first argument, followed by
@@ -949,36 +1043,32 @@ class ops(with_metaclass(OpsMeta, object)):
         Please note that there is no need to register *simple* functions as in the particular
         example above as most of them are just composite operations whose derivatives are already
         known.
+
+        To comply with numpy's ufuncs (https://numpy.org/neps/nep-0013-ufunc-overrides.html) that
+        are dispatched by :py:meth:`Number.__array_ufunc__`, an operation might register the *ufunc*
+        object that it handles. When *ufunc* is a string, it is interpreted as a name of a numpy
+        function. It can also be a list to signalize that it handles more than one function.
         """
+        # prepare ufuncs
+        ufuncs = []
+        if ufunc is not None:
+            for u in (ufunc if isinstance(ufunc, (list, tuple)) else [ufunc]):
+                if isinstance(u, string_types):
+                    if not HAS_NUMPY:
+                        continue
+                    u = getattr(np, u)
+                ufuncs.append(u)
+
         def register(function):
-            op = Operation(function, name=name)
+            op = Operation(function, name=name, ufuncs=ufuncs)
 
-            @functools.wraps(function)
-            def wrapper(num, *args, **kwargs):
-                if op.derivative is None:
-                    raise Exception("cannot run operation '{}', no derivative registered".format(
-                        op.name))
-
-                # ensure we deal with a number instance
-                num = ensure_number(num)
-
-                # apply to the nominal value
-                nominal = op.function(num.nominal, *args, **kwargs)
-
-                # apply to all uncertainties via
-                # unc_f = derivative_f(x) * unc_x
-                x = abs(op.derivative(num.nominal, *args, **kwargs))
-                uncertainties = {}
-                for name in num.uncertainties.keys():
-                    up, down = num.get_uncertainty(name)
-                    uncertainties[name] = (x * up, x * down)
-
-                # create and return the new number
-                return num.__class__(nominal, uncertainties)
-
-            # actual registration
+            # save as class attribute and also in _instances
             cls._instances[op.name] = op
-            setattr(cls, op.name, staticmethod(wrapper))
+            setattr(cls, op.name, op)
+
+            # add ufuncs to mapping
+            for ufunc in op.ufuncs:
+                cls._ufuncs[ufunc] = op.name
 
             return op
 
@@ -1001,12 +1091,40 @@ class ops(with_metaclass(OpsMeta, object)):
         """
         return cls.get_operation(*args, **kwargs)
 
+    @classmethod
+    def get_ufunc_operation(cls, ufunc):
+        """
+        Returns an operation that was previously registered to handle a NumPy *ufunc*, which can be
+        a string or the function itself. *None* is returned when no operation was found to handle
+        the function.
+        """
+        if isinstance(ufunc, string_types):
+            if not HAS_NUMPY:
+                return None
+            ufunc = getattr(np, ufunc)
+
+        if ufunc not in cls._ufuncs:
+            return None
+
+        op_name = cls._ufuncs[ufunc]
+        return cls.get_operation(op_name)
+
+    @classmethod
+    def rebuilt_ufunc_cache(cls):
+        """
+        Rebuilts the internal cache of ufuncs.
+        """
+        cls._ufuncs.clear()
+        for name, op in cls._instances.items():
+            for ufunc in op.ufuncs:
+                cls._ufuncs[ufunc] = name
+
 
 #
-# Pre-registered operations.
+# pre-registered operations
 #
 
-@ops.register
+@ops.register(ufunc="add")
 def add(x, n):
     """ add(x, n)
     Addition function.
@@ -1019,7 +1137,7 @@ def add(x, n):
     return 1.
 
 
-@ops.register
+@ops.register(ufunc="subtract")
 def sub(x, n):
     """ sub(x, n)
     Subtraction function.
@@ -1032,7 +1150,7 @@ def sub(x, n):
     return 1.
 
 
-@ops.register
+@ops.register(ufunc="multiply")
 def mul(x, n):
     """ mul(x, n)
     Multiplication function.
@@ -1045,7 +1163,7 @@ def mul(x, n):
     return n
 
 
-@ops.register
+@ops.register(ufunc="divide")
 def div(x, n):
     """ div(x, n)
     Division function.
@@ -1058,7 +1176,7 @@ def div(x, n):
     return 1. / n
 
 
-@ops.register
+@ops.register(ufunc="power")
 def pow(x, n):
     """ pow(x, n)
     Power function.
@@ -1071,7 +1189,7 @@ def pow(x, n):
     return n * x**(n - 1.)
 
 
-@ops.register
+@ops.register(ufunc="exp")
 def exp(x):
     """ exp(x)
     Exponential function.
@@ -1082,7 +1200,7 @@ def exp(x):
 exp.derivative = exp.function
 
 
-@ops.register
+@ops.register(ufunc="log")
 def log(x, base=None):
     """ log(x, base=e)
     Logarithmic function.
@@ -1105,7 +1223,33 @@ def log(x, base=None):
         return 1. / (x * infer_math(x).log(base))
 
 
-@ops.register
+@ops.register(ufunc="log10")
+def log10(x):
+    """ log10(x)
+    Logarithmic function with base 10.
+    """
+    return log.function(x, base=10.)
+
+
+@log10.derive
+def log10(x):
+    return log.derivative(x, base=10.)
+
+
+@ops.register(ufunc="log2")
+def log2(x):
+    """ log2(x)
+    Logarithmic function with base 2.
+    """
+    return log.function(x, base=2.)
+
+
+@log2.derive
+def log2(x):
+    return log.derivative(x, base=2.)
+
+
+@ops.register(ufunc="sqrt")
 def sqrt(x):
     """ sqrt(x)
     Square root function.
@@ -1118,7 +1262,7 @@ def sqrt(x):
     return 1. / (2 * infer_math(x).sqrt(x))
 
 
-@ops.register
+@ops.register(ufunc="sin")
 def sin(x):
     """ sin(x)
     Trigonometric sin function.
@@ -1131,7 +1275,7 @@ def sin(x):
     return infer_math(x).cos(x)
 
 
-@ops.register
+@ops.register(ufunc="cos")
 def cos(x):
     """ cos(x)
     Trigonometric cos function.
@@ -1144,7 +1288,7 @@ def cos(x):
     return -infer_math(x).sin(x)
 
 
-@ops.register
+@ops.register(ufunc="tan")
 def tan(x):
     """ tan(x)
     Trigonometric tan function.
@@ -1157,7 +1301,7 @@ def tan(x):
     return 1. / infer_math(x).cos(x)**2.
 
 
-@ops.register
+@ops.register(ufunc="arcsin")
 def asin(x):
     """ asin(x)
     Trigonometric arc sin function.
@@ -1174,7 +1318,7 @@ def asin(x):
     return 1. / infer_math(x).sqrt(1 - x**2.)
 
 
-@ops.register
+@ops.register(ufunc="arccos")
 def acos(x):
     """ acos(x)
     Trigonometric arc cos function.
@@ -1191,7 +1335,7 @@ def acos(x):
     return -1. / infer_math(x).sqrt(1 - x**2.)
 
 
-@ops.register
+@ops.register(ufunc="arctan")
 def atan(x):
     """ tan(x)
     Trigonometric arc tan function.
@@ -1208,7 +1352,7 @@ def atan(x):
     return 1. / (1 + x**2.)
 
 
-@ops.register
+@ops.register(ufunc="sinh")
 def sinh(x):
     """ sinh(x)
     Hyperbolic sin function.
@@ -1221,7 +1365,7 @@ def sinh(x):
     return infer_math(x).cosh(x)
 
 
-@ops.register
+@ops.register(ufunc="cosh")
 def cosh(x):
     """ cosh(x)
     Hyperbolic cos function.
@@ -1234,7 +1378,7 @@ def cosh(x):
     return infer_math(x).sinh(x)
 
 
-@ops.register
+@ops.register(ufunc="tanh")
 def tanh(x):
     """ tanh(x)
     Hyperbolic tan function.
@@ -1247,7 +1391,7 @@ def tanh(x):
     return 1. / infer_math(x).cosh(x)**2.
 
 
-@ops.register
+@ops.register(ufunc="arcsinh")
 def asinh(x):
     """ asinh(x)
     Hyperbolic arc sin function.
@@ -1259,7 +1403,7 @@ def asinh(x):
         return _math.arcsinh(x)
 
 
-@ops.register
+@ops.register(ufunc="arccosh")
 def acosh(x):
     """ acosh(x)
     Hyperbolic arc cos function.
@@ -1275,7 +1419,7 @@ asinh.derivative = acosh.function
 acosh.derivative = asinh.function
 
 
-@ops.register
+@ops.register(ufunc="arctanh")
 def atanh(x):
     """ atanh(x)
     Hyperbolic arc tan function.
@@ -1292,61 +1436,18 @@ def atanh(x):
     return 1. / (1. - x**2.)
 
 
+#
 # helper functions
+#
 
-_op_map = {
-    "+": operator.add,
-    "-": operator.sub,
-    "*": operator.mul,
-    "/": operator.truediv,
-    "**": operator.pow,
-}
-
-_op_map_reverse = dict(zip(_op_map.values(), _op_map.keys()))
-
-
-def combine_uncertainties(op, unc1, unc2, nom1=None, nom2=None, rho=0.):
+def try_float(x):
     """
-    Combines two uncertainties *unc1* and *unc2* according to an operator *op* which must be either
-    ``"+"``, ``"-"``, ``"*"``, ``"/"``, or ``"**"``. The three latter operators require that you
-    also pass the nominal values *nom1* and *nom2*, respectively. The correlation can be configured
-    via *rho*.
+    Tries to convert a value *x* to float and returns it on success, and *None* otherwise.
     """
-    # operator valid?
-    if op in _op_map:
-        f = _op_map[op]
-    elif op in _op_map_reverse:
-        f = op
-        op = _op_map_reverse[op]
-    else:
-        raise ValueError("unknown operator: {}".format(op))
-
-    # prepare values for combination, depends on operator
-    if op in ("*", "/", "**"):
-        if nom1 is None or nom2 is None:
-            raise ValueError("operator '{}' requires nominal values".format(op))
-        # numpy-safe conversion to float
-        nom1 *= 1.
-        nom2 *= 1.
-        # convert uncertainties to relative values, taking into account zeros
-        if unc1 or nom1:
-            unc1 /= nom1
-        if unc2 or nom2:
-            unc2 /= nom2
-        # determine
-        nom = abs(f(nom1, nom2))
-    else:
-        nom = 1.
-
-    # combined formula
-    if op == "**":
-        return nom * abs(nom2) * (unc1**2. + (math.log(nom1) * unc2)**2. + 2 * rho *
-            math.log(nom1) * unc1 * unc2)**0.5
-    else:
-        # flip rho for sub and div
-        if op in ("-", "/"):
-            rho = -rho
-        return nom * (unc1**2. + unc2**2. + 2. * rho * unc1 * unc2)**0.5
+    try:
+        return float(x)
+    except:
+        return None
 
 
 def ensure_number(num, *args, **kwargs):
@@ -1372,6 +1473,36 @@ def is_numpy(x):
     return HAS_NUMPY and type(x).__module__ == np.__name__
 
 
+def is_ufloat(x):
+    """
+    Returns *True* when the "uncertainties" package is available on your system and *x* is a
+    ``ufloat``.
+    """
+    return HAS_UNCERTAINTIES and isinstance(x, _uncs.core.AffineScalarFunc)
+
+
+def parse_ufloat(x, default_tag=Number.DEFAULT):
+    """
+    Takes a ``ufloat`` object *x* from the "uncertainties" package and returns a tuple with two
+    elements containing its nominal value and a dictionary with its uncertainties. When the error
+    components of *x* contain multiple uncertainties with the same name, they are combined under the
+    assumption of full correlation. When an error component is not tagged, *default_tag* is used.
+    """
+    # store error components to be combined per tag
+    components = defaultdict(list)
+    for comp, value in x.error_components().items():
+        name = comp.tag if comp.tag is not None else default_tag
+        components[name].append((x.derivatives[comp], value))
+
+    # combine components to uncertainties, assume full correlation
+    uncertainties = {
+        name: calculate_uncertainty(terms, rho=1.)
+        for name, terms in components.items()
+    }
+
+    return x.nominal_value, uncertainties
+
+
 def infer_math(x):
     """
     Returns the numpy module when :py:func:`is_numpy` for *x* is *True*, and the math module
@@ -1486,6 +1617,122 @@ def _infer_precision(unc, sig, mag, method):
     return prec, sig, mag
 
 
+_op_map = {
+    "+": operator.add,
+    "-": operator.sub,
+    "*": operator.mul,
+    "/": operator.truediv,
+    "**": operator.pow,
+}
+
+_op_map_reverse = dict(zip(_op_map.values(), _op_map.keys()))
+
+
+def calculate_uncertainty(terms, rho=0.):
+    """
+    Generically calculates the uncertainty of a quantity that depends on multiple *terms*. Each term
+    is expected to be a 2-tuple containing the derivative and the uncertainty of the term.
+    Correlations can be defined via *rho*. When *rho* is a numner, all correlations are set to this
+    value. It can also be a mapping of a 2-tuple, the two indices of the terms to describe, to their
+    correlation coefficient. In case the indices of two terms are not included in this mapping, they
+    are assumed to be uncorrelated. Example:
+
+    .. code-block:: python
+
+        calculate_uncertainty([(3, 0.5), (4, 0.5)])
+        # uncorrelated
+        # -> 2.5
+
+        calculate_uncertainty([(3, 0.5), (4, 0.5)], rho=1)
+        # fully correlated
+        # -> 3.5
+
+        calculate_uncertainty([(3, 0.5), (4, 0.5)], rho={(0, 1): 1})
+        # fully correlated
+        # -> 3.5
+
+        calculate_uncertainty([(3, 0.5), (4, 0.5)], rho={(1, 2): 1})
+        # no rho value defined for pair (0, 1), assumes zero correlation
+        # -> 2.5
+    """
+    # sum over squaresall single terms
+    variance = sum((derivative * uncertainty)**2. for derivative, uncertainty in terms)
+
+    # add second order terms of all pairs
+    for i in range(len(terms) - 1):
+        for j in range(i + 1, len(terms)):
+            _rho = rho.get((i, j), 0.) if isinstance(rho, dict) else rho
+            variance += 2. * terms[i][0] * terms[j][0] * _rho * terms[i][1] * terms[j][1]
+
+    return variance**0.5
+
+
+def combine_uncertainties(op, unc1, unc2, nom1=None, nom2=None, rho=0.):
+    """
+    Combines two uncertainties *unc1* and *unc2* according to an operator *op* which must be either
+    ``"+"``, ``"-"``, ``"*"``, ``"/"``, or ``"**"``. The three latter operators require that you
+    also pass the nominal values *nom1* and *nom2*, respectively. The correlation can be configured
+    via *rho*.
+    """
+    # operator valid?
+    if op in _op_map:
+        f = _op_map[op]
+    elif op in _op_map_reverse:
+        f = op
+        op = _op_map_reverse[op]
+    else:
+        raise ValueError("unknown operator: {}".format(op))
+
+    # when numpy arrays, the shapes of unc and nom must match
+    if is_numpy(unc1) and is_numpy(nom1) and unc1.shape != nom1.shape:
+        raise ValueError("the shape of unc1 and nom1 must be equal, found {} and {}".format(
+            unc1.shape, nom1.shape))
+    if is_numpy(unc2) and is_numpy(nom2) and unc2.shape != nom2.shape:
+        raise ValueError("the shape of unc2 and nom2 must be equal, found {} and {}".format(
+            unc2.shape, nom2.shape))
+
+    # prepare values for combination, depends on operator
+    if op in ("*", "/", "**"):
+        if nom1 is None or nom2 is None:
+            raise ValueError("operator '{}' requires nominal values".format(op))
+        # numpy-safe conversion to float
+        nom1 *= 1.
+        nom2 *= 1.
+        # convert uncertainties to relative values, taking into account zeros
+        if is_numpy(nom1) and is_numpy(unc1):
+            unc1 = np.array(unc1)
+            non_zero = nom1 != 0
+            unc1[non_zero] = unc1[non_zero] / nom1[non_zero]
+            unc1[~non_zero] = 0.
+        elif nom1:
+            unc1 = unc1 / nom1
+        else:
+            unc1 = 0.
+        if is_numpy(nom2) and is_numpy(unc2):
+            unc2 = np.array(unc2)
+            non_zero = nom2 != 0
+            unc2[non_zero] = unc2[non_zero] / nom2[non_zero]
+            unc2[~non_zero] = 0.
+        elif nom2:
+            unc2 = unc2 / nom2
+        else:
+            unc2 = 0.
+        # determine the nominal value
+        nom = abs(f(nom1, nom2))
+    else:
+        nom = 1.
+
+    # combined formula
+    if op == "**":
+        return nom * abs(nom2) * (unc1**2. + (math.log(nom1) * unc2)**2. + 2 * rho *
+            math.log(nom1) * unc1 * unc2)**0.5
+    else:
+        # flip rho for sub and div
+        if op in ("-", "/"):
+            rho = -rho
+        return nom * (unc1**2. + unc2**2. + 2. * rho * unc1 * unc2)**0.5
+
+
 def round_uncertainty(unc, method="publication"):
     """
     Rounds an uncertainty *unc* following a specific *method* and returns a 2-tuple containing the
@@ -1546,9 +1793,9 @@ def round_uncertainty(unc, method="publication"):
 def round_value(val, unc=None, unc_down=None, method="publication"):
     """
     Rounds a number *val* with a single symmetric uncertainty *unc* or asymmetric uncertainties
-    *unc* (interpreted as *up*) and *unc_down*, and calculates the orders of their magnitudes. They
-    both can be a float or a list of floats for simultaneous evaluation. When *val* is a
-    :py:class:`Number` instance, its combined uncertainty is used instead. Returns a 3-tuple
+    *unc* (interpreted as the *up* variation) and *unc_down*, and calculates the orders of their
+    magnitudes. They both can be a float or a list of floats for simultaneous evaluation. When *val*
+    is a :py:class:`Number` instance, its combined uncertainty is used instead. Returns a 3-tuple
     containing:
 
     - The string representation of the central value.
@@ -1572,8 +1819,6 @@ def round_value(val, unc=None, unc_down=None, method="publication"):
     if isinstance(val, Number):
         unc, unc_down = val.get_uncertainty()
         val = val.nominal
-    elif unc is None:
-        raise ValueError("unc must be set when val is not a Number instance")
 
     # prepare unc values
     asym = unc_down is not None
@@ -1582,18 +1827,19 @@ def round_value(val, unc=None, unc_down=None, method="publication"):
         unc_down = unc_up
 
     if not is_numpy(val):
-        # treat as lists for simultaneous rounding when not numpy arrays
+        # treat as lists for simultaneous rounding
         passed_list = isinstance(unc_up, (list, tuple)) or isinstance(unc_down, (list, tuple))
         unc_up = make_list(unc_up)
         unc_down = make_list(unc_down)
 
         # sanity checks
         if len(unc_up) != len(unc_down):
-            raise ValueError("uncertainties should have same length when passed as lists")
-        elif any(unc < 0 for unc in unc_up):
-            raise ValueError("up uncertainties must be positive: {}".format(unc_up))
-        elif any(unc < 0 for unc in unc_down):
-            raise ValueError("down uncertainties must be positive: {}".format(unc_down))
+            raise ValueError("uncertainties should have same length when passed as sequences")
+        for u in unc_up + unc_down:
+            if not try_float(u):
+                raise TypeError("uncertainties must be convertible to float: {}".format(u))
+            if u < 0:
+                raise ValueError("uncertainties must be positive: {}".format(u))
 
         # to determine the precision, use the uncertainty with the smallest magnitude
         ref_mag = min(round_uncertainty(u, method=method)[1] for u in unc_up + unc_down)
@@ -1610,11 +1856,23 @@ def round_value(val, unc=None, unc_down=None, method="publication"):
             return (val_str, [up_strs[0], down_strs[0]] if asym else [up_strs[0]], ref_mag)
 
     else:
+        # check uncertainties and cast to arrays when plain numbers
+        if not is_numpy(unc_up):
+            if not try_float(unc_up):
+                raise TypeError("uncertainty is neither an array nor float compatible: {}".format(
+                    unc_up))
+            unc_up = unc_up * np.ones_like(val)
+        if not is_numpy(unc_down):
+            if not try_float(unc_down):
+                raise TypeError("uncertainty is neither an array nor float compatible: {}".format(
+                    unc_down))
+            unc_down = unc_down * np.ones_like(val)
+
         # sanity checks
         if (unc_up < 0).any():
-            raise ValueError("up uncertainties must be positive: {}".format(unc_up))
+            raise ValueError("uncertainties must be positive: {}".format(unc_up))
         elif (unc_down < 0).any():
-            raise ValueError("down uncertainties must be positive: {}".format(unc_down))
+            raise ValueError("uncertainties must be positive: {}".format(unc_down))
 
         # to determine the precision, use the uncertainty with the smallest magnitude
         ref_mag_up = round_uncertainty(unc_up, method=method)[1]
